/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * @file
 * qp Statemachine, works as interface between library calls and ds485
 * Protocol stack. Handles the status messages and status request.
 */
#include "config.h"
#include "mmi.h"
#include "unix/bsp.h"
#include "digitalSTROM/ds485.h"
#include "ds485-qp.h"
#include "ds485-signals.h"
#include "unix/ds485-unix.h"

Q_DEFINE_THIS_MODULE(mmi)

extern Ds485LibInst_t  devInst;

#define MMI_TICK_TIMEOUT_TICKS                      (250/BSP_MS_PER_TICKS)

/* Active object class -----------------------------------------------------*/
typedef struct MMITag {
    QActive super;
    bus_state_t busState;
} MMI;

/* Local objects -----------------------------------------------------------*/
static MMI l_mmi;   /* the single instance of the mmi active object */

/* Global-scope objects ----------------------------------------------------*/
QActive * const AO_MMI = (QActive *)&l_mmi;      /* "opaque" AO pointer */

static void setBusState( MMI *me, bus_state_t state )
{
    if (me->busState == state) {
        return;
    }
    me->busState = state;
    if (devInst.state != NULL) {
        devInst.state(&devInst, state);
    }
}

/* Local methods -----------------------------------------------------------*/
static QState MMI_initial(MMI *me, QEvent const *e);
static QState MMI_top(MMI *me, QEvent const *e);

/*..........................................................................*/
void MMI_ctor(void)
{
    MMI *me = &l_mmi;
    /* call active object (superclass) constructor */
    QActive_ctor(&me->super, (QStateHandler)&MMI_initial);
    me->busState = DS485_STATE_ISOLATED;
}

/*..........................................................................*/
static QState MMI_initial(MMI *me, QEvent const *e)
{
    /* QS dictionary entries */
    QS_OBJ_DICTIONARY(&l_mmi);
    QS_FUN_DICTIONARY(&MMI_top);
    return Q_TRAN(&MMI_top);
}

/*..........................................................................*/
static QState MMI_top(MMI *me, QEvent const *e)
{
    switch (e->sig) {
    case MMI_DS485_DEVICE_REGISTERED_SIG:
        return Q_HANDLED();

    case MMI_DS485_DEVICE_LOST_SIG:
        return Q_HANDLED();

    case MMI_DS485_STATE_SIG:
        switch (((MMIDS485StateEvt const *)e)->ds485) {
        case MMI_DS485_INITIALIZING:
            setBusState( me, DS485_STATE_ISOLATED );
            break;
        case MMI_DS485_READY:
            setBusState( me, DS485_STATE_ACTIVE );
            break;
        case MMI_DS485_JOINING:
            setBusState( me, DS485_STATE_JOIN );
            break;
        }
        return Q_HANDLED();

    case DS485_STOPPED_SIG:
        setBusState( me, DS485_STATE_ISOLATED );
        return Q_HANDLED();

    case DS485_ADDRESS_SIG:
        setBusState( me, DS485_STATE_CONNECTED );
        return Q_HANDLED();

    case MMI_DS485_DSUID_REGISTRED_SIG: {
        MMI_DS485DsuidRegisteredEvt  *rxe = (MMI_DS485DsuidRegisteredEvt *)e;
        if (devInst.busChange != NULL) {
            devInst.busChange(&devInst, &rxe->dsuid, 0);
        }
        return Q_HANDLED();
    }

    case MMI_DS485_DSUID_DEREGISTRED_SIG: {
        MMI_DS485DsuidRegisteredEvt *rxe = (MMI_DS485DsuidRegisteredEvt *)e;
        if (devInst.busChange != NULL) {
            devInst.busChange(&devInst, &rxe->dsuid, 1);
        }
        return Q_HANDLED();
    }

    case MMI_REQ_BUS_STATE_SIG: {
        MMI_BusStateEvt *evt = Q_NEW(MMI_BusStateEvt, MMI_RES_BUS_STATE_SIG);
        evt->state  = me->busState;
        ccSendResponse(e, (QEvent*)evt);

        return Q_HANDLED();
    }
    }

    return Q_SUPER(&QHsm_top);
}
